#!/usr/bin/env python

SERVER_ADDRESS = '127.0.0.1'
SERVER_PORT = 8888

import os, sys, pygtk, gobject, gtk, gtk.glade, time, threading, SocketServer
import xmlrpclib, thread
import gtk.gdk as gdk
import logging
logging.logMultiprocessing = 0 # fix giant annoying logging bug.

logging.basicConfig(level=logging.DEBUG)

pygtk.require("2.0")

import SimpleXMLRPCServer
scriptdir = os.path.abspath(os.path.dirname(sys.argv[0]))
for d in ("lib", "lib/chardet", "sites", "connectors"):
        sys.path.append(os.path.join(scriptdir, d)) 
os.chdir(scriptdir)

from Googlyrics import *

class ConnectorThread(threading.Thread):
    def __init__(self, conn):
        threading.Thread.__init__(self)
        self.conn_module = conn
    def run(self):
        mod = __import__(self.conn_module)
        mod.run()

class GoodLyricsUI:
    def __init__(self, rpc):
        config = ConfigParser()
        config.read("config.ini")
        connector = config.get("GoodLyrics", "connector")
        ct = ConnectorThread(connector)
        ct.start()
        #Set the Glade file
        self.gladefile = "goodlyrics.glade"
        self.wTree = gtk.glade.XML(self.gladefile) 
        #Get the Main Window, and connect the "destroy" event
        self.window = self.wTree.get_widget("window1")
        self.wTree.get_widget("vbox1").set_focus_chain([
            self.wTree.get_widget("toolbar1"),
            #self.wTree.get_widget("backbutton"), 
            #self.wTree.get_widget("nextbutton"), 
            #self.wTree.get_widget("stopbutton"), 
            self.wTree.get_widget("artistentry"),
            self.wTree.get_widget("titleentry"),
            self.wTree.get_widget("toolbutton5"),
            #self.wTree.get_widget("lyricsview"), 
            ])
        #self.wTree.get_widget("toolbar1").set_focus_chain([])
        self.wTree.get_widget("stopbutton").set_sensitive(False)
        
        self.window.show()
        self.window.connect("destroy", self.kill)
        
        dic = {
               "goback"      : self.prevLyrics,
               "gonext"      : self.nextLyrics,
               "gobutton"    : self.readEntryLyrics,
               "stopclicked" : self.doStop
        }
        
        self.wTree.signal_autoconnect(dic)
        self.lyricbuffer = gtk.TextBuffer()
        self.wTree.get_widget("lyricsview").set_buffer(self.lyricbuffer)
        rpc.ui = self
        self.rpc = rpc
    def doStop(self, data):
        thread.start_new_thread(self.rpc.stopLoading, ())
    def readEntryLyrics(self, data):
        artist = self.wTree.get_widget("artistentry").get_text()
        title = self.wTree.get_widget("titleentry").get_text()
        thread.start_new_thread(self.rpc.showLyrics, (artist, title))
    def prevLyrics(self, data):
        thread.start_new_thread(self.rpc.prevLyrics, ())
    def nextLyrics(self, data):
        thread.start_new_thread(self.rpc.nextLyrics, ())
    def kill(self, data):
        os._exit(1)

class RPCLyrics:
    ui = None
    def __init__(self):
        self.g = Googlyrics()    
        self.showLock = threading.Lock()
        self.lyricsLock = threading.Lock()
        self.ownerLock = threading.Lock()
        self.owner = None
    def showLyrics(self, artist, title):
        # this must be run in a seperate thread or the app will hang waiting for
        # the XMLRPC reply! Damn gobject only having a single mainloop -_-
        if title.startswith("http://"):
            return True
        p = threading.Thread(target=self._showLyrics, args=(artist, title,))
        p.start()
        return True
    def isOwner(self):
        self.ownerLock.acquire()
        val = (self.owner == threading.currentThread())
        self.ownerLock.release()
        return val
    def _showLyrics(self, artist, title):
        self.showLock.acquire()
        self.ownerLock.acquire()
        self.owner = threading.currentThread()
        self.ownerLock.release()
        gobject.idle_add(self.setEnabledButtons, False, False)
        if (self.isOwner()):
            gobject.idle_add(self.setLyrics, "Loading...", artist, title)
        self.starttime = time.time()
        try:
            self.lyiter = self.g.find_lyrics(title, artist)
            self.outlyric = []
            try:
                res = self.lyiter.next()
                if not self.isOwner(): return
                self.outlyric.append(res)
            except StopIteration:
                logging.debug("Stopped iteration, no more Google results.")
        except Exception, e:
            if not self.isOwner(): return
            gobject.idle_add(self.setMessage, "Google query error:\n" + str(e))
            self.showLock.release()
            return False
        self.ourlyric_cache = {}
        self.lynum = 0
        if not self.isOwner(): return
        if len(self.outlyric) == 0:
            gobject.idle_add(self.setLyrics, "No lyrics for track found", artist, title)
            ret = False
        else:
            self.loadly()
            ret = True
        if not self.isOwner(): return
        self.showLock.release()
        return ret
    def nextLyrics(self):
        self.starttime = time.time()
        self.lynum += 1
        self.loadly()
        return True
    def prevLyrics(self):
        self.starttime = time.time()
        self.lynum -= 1
        self.loadly()
        return True
    def setEnabledButtons(self, next, back):
        self.ui.wTree.get_widget("nextbutton").set_sensitive(next)
        self.ui.wTree.get_widget("backbutton").set_sensitive(back)
        if ((next == False) and (back == False)):
            self.ui.wTree.get_widget("stopbutton").set_sensitive(True)
        else:
            self.ui.wTree.get_widget("stopbutton").set_sensitive(False)
    def loadly(self):
        self.lyricsLock.acquire()
        self.ownerLock.acquire()
        self.owner = threading.currentThread()
        self.ownerLock.release()
        if self.lynum < 0: 
            self.lynum = 0
            if not self.isOwner(): return
            self.lyricsLock.release()
            return
        if self.isOwner():
            gobject.idle_add(self.setMessage, "Loading...")
            gobject.idle_add(self.setEnabledButtons, False, False)
        if self.lynum >= len(self.outlyric):
            try:
                logging.debug("Appending...")
                res = self.lyiter.next()
                if not self.isOwner(): return
                self.outlyric.append(res)
                logging.debug("Done append.")
            except StopIteration:
                if not self.isOwner(): return
                if len(self.outlyric) == 0:
                        gobject.idle_add(self.setMessage, "No lyrics for track found.")
                        gobject.idle_add(self.setEnabledButtons, False, False)
                else:
                    if (self.isOwner()):
                        gobject.idle_add(self.setMessage, "No more results.")
                        gobject.idle_add(self.setEnabledButtons, False, True)
                if not self.isOwner(): return
                self.lyricsLock.release()
                return
        try:
            logging.debug("Requested: %d" % self.lynum)
            if not self.lynum in self.ourlyric_cache:
                logging.debug("Fetching: %d" % self.lynum)
                if not self.isOwner(): return
                self.ourlyric_cache[self.lynum] = self.outlyric[self.lynum].getLyric()
                if not self.isOwner(): return
                self.ourlyric_cache[self.lynum].timegrab = time.time() - self.starttime
            lyric = self.ourlyric_cache[self.lynum]
            if not self.isOwner(): return
            gobject.idle_add(self.setLyrics, "%s\n\n##########\n\nFetched in %f seconds\nFetched from %s" % (lyric.lyrics, lyric.timegrab, lyric.url), lyric.artist, lyric.title)
        except IndexError:
            pass
        except AttributeError, e:
            print "AttributeError: %s" % str(e)
            del self.outlyric[self.lynum]
            if self.lynum in self.ourlyric_cache: 
                del self.ourlyric_cache[self.lynum]
            if not self.isOwner(): return
            self.lyricsLock.release()
            self.loadly()
            return
        if not self.isOwner(): return
        if self.lynum == 0:
            gobject.idle_add(self.setEnabledButtons, True, False)
        else:
            gobject.idle_add(self.setEnabledButtons, True, True)
        self.lyricsLock.release()
        return True
    def setMessage(self, message):
        self.ui.lyricbuffer.set_text(message)
    def setLyrics(self, lyricstr, artist, title):
        logging.debug("setting to:" + '\n'.join(lyricstr.split('\n')[:5]))
        self.ui.lyricbuffer.set_text(lyricstr)
        self.ui.window.set_title(title + " by " + artist + " - GoodLyrics")
        self.ui.wTree.get_widget("artistentry").set_text(artist)
        self.ui.wTree.get_widget("titleentry").set_text(title)
        return False
    def stopLoading(self):
        try:
            self.lyricsLock.release()
        except:
            pass
        try:
            self.showLock.release()
        except: 
            pass
        self.setEnabledButtons(True, True)
        self.owner = None
        gobject.idle_add(self.setMessage, "Stopped.")

class RPCServerThread(threading.Thread):
    "RPC server-proxy for RPCLyrics"
    rpcobj = None
    def run(self):
        server = SimpleXMLRPCServer.SimpleXMLRPCServer((SERVER_ADDRESS, SERVER_PORT))
        server.register_instance(self.rpcobj)
        logging.info("GoodLyrics XMLRPC server listening on %s:%d" % (SERVER_ADDRESS, SERVER_PORT))
        server.serve_forever()

if __name__ == "__main__":
    gdk.threads_init()
    rpcobj = RPCLyrics()
    srv = RPCServerThread()
    srv.rpcobj = rpcobj
    srv.start()
    gui = GoodLyricsUI(rpcobj)
    try:
        gtk.main()
    except KeyboardInterrupt:
        os._exit(1)
