#!/usr/bin/env python

import dbus
import re
import xmlrpclib
import time
import gobject
import logging
import sys
logging.logMultiprocessing = 0 # fix giant annoying logging bug.

from multiprocessing import Process
from dbus.mainloop.glib import DBusGMainLoop

logger = logging.getLogger("connector")

dbus_loop = DBusGMainLoop()
server = xmlrpclib.ServerProxy("http://localhost:8888")
dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)
bus = dbus.SessionBus()
org_mpris_re = re.compile('^org\.mpris\.([^.]+)$')

def possible_names():
    return [ name for name in bus.list_names() if org_mpris_re.match(name) ]

# returns first matching player - from mpris-remote, thanks!
def get_player():
    names = possible_names()
    if not names:
        logging.fatal("No MPRIS-compliant player found running.")
        raise SystemExit(1)
    return org_mpris_re.match(names[0]).group(1)

def handler(nfo):
    logger.debug("got signal!")
    if nfo.has_key('title'):
        try:
            server.showLyrics(str(nfo['artist']), str(nfo['title']))
        except KeyError:
            server.showLyrics("", str(nfo['title']))

def run():
    player_name = get_player()
    logger.info("Found %s" % player_name)
    obj = bus.get_object('org.mpris.' + player_name, '/Player')
    mtd = obj.get_dbus_method('GetMetadata', 'org.freedesktop.MediaPlayer')
    time.sleep(0.5)
    handler(mtd())
    time.sleep(0.5)
    bus.add_signal_receiver(handler, dbus_interface = "org.freedesktop.MediaPlayer", signal_name = "TrackChange")
    loop = gobject.MainLoop()
    gobject.threads_init()
    context = loop.get_context()
    while True:
        context.iteration(True)

if __name__ == "__main__":
    run()
