#!/usr/bin/env python
# -*- coding: utf-8 -*-

from LyricSearchResult import LyricSearchResult
from ConfigParser import ConfigParser
import sys, os, re, urllib, threading
import logging
from GoogleQuery import GoogleQuery
import cookielib

class Googlyrics:
    plugins = []
    def __init__(self):
        config = ConfigParser()
        self.cookies = cookielib.LWPCookieJar("cookies.txt")
        try:
            self.cookies.load()
        except:
            pass
        config.read("config.ini")
        plugs = config.get("Googlyrics", "plugins").split(" ")
        loaded = 0
        for plug in plugs:
            exec "from " + plug +" import *"
            exec "self.plugins.append(" + plug + "())"
            loaded += 1
    def clean(self,instr):
        instr = re.sub(r'\(.+\)', "", instr)
        instr = re.sub(r'\[.+\]', "", instr)
        return instr
    def find_lyrics(self, title, artist):
        qs = [
            u'intitle:"%(artist)s - %(title)s" lyrics',
            u'"%(title)s" "%(artist)s" lyrics',
            u'%(title)s %(artist)s lyrics',
            u'%(title)s %(artist)s',
            ]
        for i in range(len(qs)):
            qs.append(qs[i] % {'title': self.clean(title), 'artist':self.clean(artist)})
            qs[i] = qs[i] % {'title': title, 'artist':artist}
        threads = []
        for q in qs:
            for numres in [100]:
                current = GooglyricsThreadFetcher(self.plugins, title, artist, q, self.cookies, numres)
                threads.append(current)
        results_history=[]
        for thread in threads:
            thread.run()
            for result in thread.lyrics:
                if result in results_history: continue
                results_history.append(result)
                yield result
    def test_all(self):
        passlist = []
        faillist = []
        for plug in self.plugins:
            sys.stdout.write("Test of " + plug.name)
            if plug.url_match(plug.testurl):
                if plug.selftest():
                    print " passed"
                    passlist.append(plug.name)
                else:
                    print " failed"
                    faillist.append(plug.name)
            else:
                print plug.name + " rejected test url"
        print "PASSED:"
        for a in passlist:
            print a
        print "FAILED:"
        for a in faillist:
            print a

class GooglyricsThreadFetcher(threading.Thread):
    def __init__(self, plugins, title, artist, query, cookies, numres = 100):
        threading.Thread.__init__(self)
        self.plugins = plugins
        self.numres = numres
        self.title, self.artist = title, artist
        self.query = query
        self.cookies = cookies
        logging.debug("Inited")
    def run(self):
        self.lyrics = []
        logging.debug("Making query: %s" % self.query)
        query = GoogleQuery(self.query, self.cookies, self.numres)
        for url in query.get_all_urls():
            for plug in self.plugins:
                if plug.url_match(url):
                    self.lyrics.append(LyricSearchResult(plug, url, self.title, self.artist))                    
        


if __name__ == '__main__':
    sys.path.append("./lib/")
    sys.path.append("./sites/")
    sys.path.append("./lib/chardet/")
    g = Googlyrics()
    g.test_all()
